@@ -86,7 +86,7 @@ char *blkid_get_cache_filename(struct blkid_config *conf)
  * @cache: pointer to return cache handler
  * @filename: path to the cache file or NULL for the default path
  *
- * Allocates and initialize librray cache handler.
+ * Allocates and initializes library cache handler.
  *
  * Returns: 0 on success or number less than zero in case of error.
  */
@@ -99,12 +99,10 @@ int blkid_get_cache(blkid_cache *ret_cache, const char *filename)
 
 	blkid_init_debug(0);
 
-	DBG(CACHE, blkid_debug("creating blkid cache (using %s)",
-				filename ? filename : "default cache"));
-
-	if (!(cache = (blkid_cache) calloc(1, sizeof(struct blkid_struct_cache))))
+	if (!(cache = calloc(1, sizeof(struct blkid_struct_cache))))
 		return -BLKID_ERR_MEM;
 
+	DBG(CACHE, ul_debugobj(cache, "alloc (from %s)", filename ? filename : "default cache"));
 	INIT_LIST_HEAD(&cache->bic_devs);
 	INIT_LIST_HEAD(&cache->bic_tags);
 
@@ -133,9 +131,9 @@ void blkid_put_cache(blkid_cache cache)
 
 	(void) blkid_flush_cache(cache);
 
-	DBG(CACHE, blkid_debug("freeing cache struct"));
+	DBG(CACHE, ul_debugobj(cache, "freeing cache struct"));
 
-	/* DBG(CACHE, blkid_debug_dump_cache(cache)); */
+	/* DBG(CACHE, ul_debug_dump_cache(cache)); */
 
 	while (!list_empty(&cache->bic_devs)) {
 		blkid_dev dev = list_entry(cache->bic_devs.next,
@@ -144,6 +142,7 @@ void blkid_put_cache(blkid_cache cache)
 		blkid_free_dev(dev);
 	}
 
+	DBG(CACHE, ul_debugobj(cache, "freeing cache tag heads"));
 	while (!list_empty(&cache->bic_tags)) {
 		blkid_tag tag = list_entry(cache->bic_tags.next,
 					   struct blkid_struct_tag,
@@ -154,7 +153,7 @@ void blkid_put_cache(blkid_cache cache)
 						   struct blkid_struct_tag,
 						   bit_names);
 
-			DBG(CACHE, blkid_debug("warning: unfreed tag %s=%s",
+			DBG(CACHE, ul_debugobj(cache, "warning: unfreed tag %s=%s",
 						bad->bit_name, bad->bit_val));
 			blkid_free_tag(bad);
 		}
@@ -184,11 +183,11 @@ void blkid_gc_cache(blkid_cache cache)
 	list_for_each_safe(p, pnext, &cache->bic_devs) {
 		blkid_dev dev = list_entry(p, struct blkid_struct_dev, bid_devs);
 		if (stat(dev->bid_name, &st) < 0) {
-			DBG(CACHE, blkid_debug("freeing %s", dev->bid_name));
+			DBG(CACHE, ul_debugobj(cache, "freeing non-existing %s", dev->bid_name));
 			blkid_free_dev(dev);
 			cache->bic_flags |= BLKID_BIC_FL_CHANGED;
 		} else {
-			DBG(CACHE, blkid_debug("Device %s exists", dev->bid_name));
+			DBG(CACHE, ul_debug("Device %s exists", dev->bid_name));
 		}
 	}
 }
