@@ -37,7 +37,10 @@
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <linux/fs.h>
-#include <blkid.h>
+
+#ifdef HAVE_LIBBLKID
+# include <blkid.h>
+#endif
 
 #include "nls.h"
 #include "strutils.h"
@@ -107,6 +110,7 @@ static void __attribute__((__noreturn__)) usage(void)
 	exit(EXIT_SUCCESS);
 }
 
+#ifdef HAVE_LIBBLKID
 /*
  * Check existing signature on the open fd
  * Returns	0  signature found
@@ -142,6 +146,7 @@ out:
 	blkid_free_probe(pr);
 	return ret;
 }
+#endif /* HAVE_LIBBLKID */
 
 int main(int argc, char **argv)
 {
@@ -149,7 +154,7 @@ int main(int argc, char **argv)
 	int c, fd, verbose = 0, secsize, force = 0;
 	uint64_t end, blksize, step, range[2], stats[2];
 	struct stat sb;
-	struct timeval now, last;
+	struct timeval now = { 0 }, last = { 0 };
 	int act = ACT_DISCARD;
 
 	static const struct option longopts[] = {
@@ -252,27 +257,31 @@ int main(int argc, char **argv)
 	if (range[1] % secsize)
 		errx(EXIT_FAILURE, _("%s: length %" PRIu64 " is not aligned "
 			 "to sector size %i"), path, range[1], secsize);
-
-	 /* Check for existing signatures on the device */
-	switch(probe_device(fd, path)) {
-	case 0: /* signature detected */
-		/*
-		 * Only require force in interactive mode to avoid
-		 * breaking existing scripts
-		 */
-		if (!force && isatty(STDIN_FILENO)) {
-			errx(EXIT_FAILURE,
-			     _("This is destructive operation, data will " \
-			       "be lost! Use the -f option to override."));
-		}
+#ifdef HAVE_LIBBLKID
+	if (force)
 		warnx(_("Operation forced, data will be lost!"));
-		break;
-	case 1: /* no signature */
-		break;
-	default: /* error */
-		err(EXIT_FAILURE, _("failed to probe the device"));
-		break;
+	else {
+		 /* Check for existing signatures on the device */
+		switch(probe_device(fd, path)) {
+		case 0: /* signature detected */
+			/*
+			 * Only require force in interactive mode to avoid
+			 * breaking existing scripts
+			 */
+			if (isatty(STDIN_FILENO)) {
+				errx(EXIT_FAILURE,
+				     _("This is destructive operation, data will " \
+				       "be lost! Use the -f option to override."));
+			}
+			break;
+		case 1: /* no signature */
+			break;
+		default: /* error */
+			err(EXIT_FAILURE, _("failed to probe the device"));
+			break;
+		}
 	}
+#endif /* HAVE_LIBBLKID */
 
 	stats[0] = range[0], stats[1] = 0;
 	gettime_monotonic(&last);
