@@ -51,6 +51,8 @@
 #include <sys/utsname.h>
 #include <unistd.h>
 
+#include "c.h"
+#include "strutils.h"
 #include "nls.h"
 #include "xalloc.h"
 #include "closestream.h"
@@ -130,8 +132,8 @@ static void __attribute__((__noreturn__)) usage(void)
 int main(int argc, char **argv)
 {
 	FILE *map;
-	int proFd;
-	char *mapFile, *proFile, *mult = NULL;
+	int proFd, has_mult = 0, multiplier = 0;
+	char *mapFile, *proFile;
 	size_t len = 0, indx = 1;
 	unsigned long long add0 = 0;
 	unsigned int step;
@@ -147,6 +149,7 @@ int main(int argc, char **argv)
 	int maplineno = 1;
 	int popenMap;		/* flag to tell if popen() has been used */
 	int header_printed;
+	double rep = 0;
 
 	static const struct option longopts[] = {
 		{"mapfile", required_argument, NULL, 'm'},
@@ -198,7 +201,8 @@ int main(int argc, char **argv)
 			optInfo++;
 			break;
 		case 'M':
-			mult = optarg;
+			multiplier = strtol_or_err(optarg, _("failed to parse multiplier"));
+			has_mult = 1;
 			break;
 		case 'r':
 			optReset++;
@@ -216,14 +220,13 @@ int main(int argc, char **argv)
 		}
 	}
 
-	if (optReset || mult) {
-		int multiplier, fd, to_write;
+	if (optReset || has_mult) {
+		int fd, to_write;
 
 		/* When writing the multiplier, if the length of the
 		 * write is not sizeof(int), the multiplier is not
 		 * changed. */
-		if (mult) {
-			multiplier = strtoul(mult, NULL, 10);
+		if (has_mult) {
 			to_write = sizeof(int);
 		} else {
 			multiplier = 0;
@@ -346,7 +349,7 @@ int main(int argc, char **argv)
 			errx(EXIT_FAILURE,
 			     _("profile address out of range. Wrong map file?"));
 
-		while (indx < (next_add - add0) / step) {
+		while (step > 0 && indx < (next_add - add0) / step) {
 			if (optBins && (buf[indx] || optAll)) {
 				if (!header_printed) {
 					printf("%s:\n", fn_name);
@@ -370,7 +373,7 @@ int main(int argc, char **argv)
 			else
 				printf("%6u %-40s %8.4f\n",
 				       this, fn_name, this / (double)fn_len);
-			if (optSub) {
+			if (optSub && step > 0) {
 				unsigned long long scan;
 
 				for (scan = (fn_add - add0) / step + 1;
@@ -396,13 +399,16 @@ int main(int argc, char **argv)
 	/* clock ticks, out of kernel text - probably modules */
 	printf("%6u %s\n", buf[len / sizeof(*buf) - 1], "*unknown*");
 
+	if (fn_add > add0)
+		rep = total / (double)(fn_add - add0);
+
 	/* trailer */
 	if (optVerbose)
 		printf("%016x %-40s %6u %8.4f\n",
-		       0, "total", total, total / (double)(fn_add - add0));
+		       0, "total", total, rep);
 	else
 		printf("%6u %-40s %8.4f\n",
-		       total, _("total"), total / (double)(fn_add - add0));
+		       total, _("total"), rep);
 
 	popenMap ? pclose(map) : fclose(map);
 	exit(EXIT_SUCCESS);
