@@ -147,7 +147,8 @@ struct script_control {
 	 isterm:1;		/* is child process running as terminal */
 };
 
-static ssize_t log_info(struct script_control *ctl, const char *name, const char *msgfmt, ...);
+static ssize_t log_info(struct script_control *ctl, const char *name, const char *msgfmt, ...)
+			__attribute__((__format__ (__printf__, 3, 4)));
 
 static void script_init_debug(void)
 {
@@ -208,7 +209,7 @@ static void __attribute__((__noreturn__)) usage(void)
 	fputs(_(" -e, --return                  return exit code of the child process\n"), out);
 	fputs(_(" -f, --flush                   run flush after each write\n"), out);
 	fputs(_("     --force                   use output file even when it is a link\n"), out);
-	fputs(_(" -E, --echo <when>             echo input (auto, always or never)\n"), out);
+	fputs(_(" -E, --echo <when>             echo input in session (auto, always or never)\n"), out);
 	fputs(_(" -o, --output-limit <size>     terminate if output files exceed size\n"), out);
 	fputs(_(" -q, --quiet                   be quiet\n"), out);
 
@@ -305,8 +306,9 @@ static int log_close(struct script_control *ctl,
 		gettime_monotonic(&now);
 		timersub(&now, &log->starttime, &delta);
 
-		log_info(ctl, "DURATION", "%ld.%06ld",
-			(long)delta.tv_sec, (long)delta.tv_usec);
+		log_info(ctl, "DURATION", "%"PRId64".%06"PRId64,
+			(int64_t)delta.tv_sec,
+			(int64_t)delta.tv_usec);
 		log_info(ctl, "EXIT_CODE", "%d", status);
 		break;
 	}
@@ -469,8 +471,8 @@ static ssize_t log_write(struct script_control *ctl,
 
 		gettime_monotonic(&now);
 		timersub(&now, &log->oldtime, &delta);
-		ssz = fprintf(log->fp, "%ld.%06ld %zd\n",
-			(long)delta.tv_sec, (long)delta.tv_usec, bytes);
+		ssz = fprintf(log->fp, "%"PRId64".%06"PRId64" %zd\n",
+			(int64_t)delta.tv_sec, (int64_t)delta.tv_usec, bytes);
 		if (ssz < 0)
 			return -errno;
 
@@ -482,9 +484,9 @@ static ssize_t log_write(struct script_control *ctl,
 
 		gettime_monotonic(&now);
 		timersub(&now, &log->oldtime, &delta);
-		ssz = fprintf(log->fp, "%c %ld.%06ld %zd\n",
+		ssz = fprintf(log->fp, "%c %"PRId64".%06"PRId64" %zd\n",
 			stream->ident,
-			(long)delta.tv_sec, (long)delta.tv_usec, bytes);
+			(int64_t)delta.tv_sec, (int64_t)delta.tv_usec, bytes);
 		if (ssz < 0)
 			return -errno;
 
@@ -518,7 +520,8 @@ static ssize_t log_stream_activity(
 	return outsz;
 }
 
-static ssize_t log_signal(struct script_control *ctl, int signum, char *msgfmt, ...)
+static ssize_t __attribute__ ((__format__ (__printf__, 3, 4)))
+	log_signal(struct script_control *ctl, int signum, const char *msgfmt, ...)
 {
 	struct script_log *log;
 	struct timeval now, delta;
@@ -548,12 +551,12 @@ static ssize_t log_signal(struct script_control *ctl, int signum, char *msgfmt,
 	}
 
 	if (*msg)
-		sz = fprintf(log->fp, "S %ld.%06ld SIG%s %s\n",
-			(long)delta.tv_sec, (long)delta.tv_usec,
+		sz = fprintf(log->fp, "S %"PRId64".%06"PRId64" SIG%s %s\n",
+			(int64_t)delta.tv_sec, (int64_t)delta.tv_usec,
 			signum_to_signame(signum), msg);
 	else
-		sz = fprintf(log->fp, "S %ld.%06ld SIG%s\n",
-			(long)delta.tv_sec, (long)delta.tv_usec,
+		sz = fprintf(log->fp, "S %"PRId64".%06"PRId64" SIG%s\n",
+			(int64_t)delta.tv_sec, (int64_t)delta.tv_usec,
 			signum_to_signame(signum));
 
 	log->oldtime = now;
@@ -673,7 +676,6 @@ static int callback_log_stream_activity(void *data, int fd, char *buf, size_t bu
 
 	ctl->outsz += ssz;
 
-
 	/* check output limit */
 	if (ctl->maxsz != 0 && ctl->outsz >= ctl->maxsz) {
 		if (!ctl->quiet)
@@ -752,7 +754,7 @@ int main(int argc, char **argv)
 		.in  = { .ident = 'I' },
 	};
 	struct ul_pty_callbacks *cb;
-	int ch, format = 0, caught_signal = 0, rc = 0, echo = 0;
+	int ch, format = 0, caught_signal = 0, rc = 0, echo = 1;
 	const char *outfile = NULL, *infile = NULL;
 	const char *timingfile = NULL, *shell = NULL, *command = NULL;
 
@@ -798,12 +800,7 @@ int main(int argc, char **argv)
 	script_init_debug();
 	ON_DBG(PTY, ul_pty_init_debug(0xFFFF));
 
-	/* The default is to keep ECHO flag when stdin is not terminal. We need
-	 * it to make stdin (in case of "echo foo | script") log-able and
-	 * visible on terminal, and for backward compatibility.
-	 */
 	ctl.isterm = isatty(STDIN_FILENO);
-	echo = ctl.isterm ? 0 : 1;
 
 	while ((ch = getopt_long(argc, argv, "aB:c:eE:fI:O:o:qm:T:t::Vh", longopts, NULL)) != -1) {
 
@@ -1003,23 +1000,23 @@ int main(int argc, char **argv)
 		time_t tvec = script_time((time_t *)NULL);
 
 		strtime_iso(&tvec, ISO_TIMESTAMP, buf, sizeof(buf));
-		log_info(&ctl, "START_TIME", buf);
+		log_info(&ctl, "START_TIME", "%s", buf);
 
 		if (ctl.isterm) {
 			init_terminal_info(&ctl);
-			log_info(&ctl, "TERM", ctl.ttytype);
-			log_info(&ctl, "TTY", ctl.ttyname);
+			log_info(&ctl, "TERM", "%s", ctl.ttytype);
+			log_info(&ctl, "TTY", "%s", ctl.ttyname);
 			log_info(&ctl, "COLUMNS", "%d", ctl.ttycols);
 			log_info(&ctl, "LINES", "%d", ctl.ttylines);
 		}
-		log_info(&ctl, "SHELL", shell);
+		log_info(&ctl, "SHELL", "%s", shell);
 		if (command)
-			log_info(&ctl, "COMMAND", command);
-		log_info(&ctl, "TIMING_LOG", timingfile);
+			log_info(&ctl, "COMMAND", "%s", command);
+		log_info(&ctl, "TIMING_LOG", "%s", timingfile);
 		if (outfile)
-			log_info(&ctl, "OUTPUT_LOG", outfile);
+			log_info(&ctl, "OUTPUT_LOG", "%s", outfile);
 		if (infile)
-			log_info(&ctl, "INPUT_LOG", infile);
+			log_info(&ctl, "INPUT_LOG", "%s", infile);
 	}
 
         /* this is the main loop */
