@@ -1113,13 +1113,18 @@ static void expand(struct more_control *ctl, char *inbuf)
 	char *outstr;
 	char c;
 	char *temp;
-	int tempsz, xtra, offset;
+	int tempsz, xtra = 0, offset;
+
+	if (!ctl->no_tty_in)
+		xtra += strlen(ctl->file_names[ctl->argv_position]) + 1;
+	if (ctl->shell_line)
+		xtra += strlen(ctl->shell_line) + 1;
 
-	xtra = strlen(ctl->file_names[ctl->argv_position]) + strlen(ctl->shell_line) + 1;
 	tempsz = COMMAND_BUF + xtra;
 	temp = xmalloc(tempsz);
 	inpstr = inbuf;
 	outstr = temp;
+
 	while ((c = *inpstr++) != '\0') {
 		offset = outstr - temp;
 		if (tempsz - offset - 1 < xtra) {
@@ -1209,7 +1214,8 @@ static void sigwinch_handler(struct more_control *ctl)
 	prepare_line_buffer(ctl);
 }
 
-static void execute(struct more_control *ctl, char *filename, char *cmd, ...)
+static void __attribute__((__format__ (__printf__, 3, 4)))
+	execute(struct more_control *ctl, char *filename, const char *cmd, ...)
 {
 	pid_t id;
 	va_list argp;
@@ -1249,12 +1255,9 @@ static void execute(struct more_control *ctl, char *filename, char *cmd, ...)
 		}
 		va_end(argp);
 
-		if (geteuid() != getuid() || getegid() != getgid()) {
-			if (setuid(getuid()) < 0)
-				err(EXIT_FAILURE, _("setuid failed"));
-			if (setgid(getgid()) < 0)
-				err(EXIT_FAILURE, _("setgid failed"));
-		}
+		if ((geteuid() != getuid() || getegid() != getgid())
+		    && drop_permissions() != 0)
+			err(EXIT_FAILURE, _("drop permissions failed"));
 
 		execvp(cmd, args);
 		errsv = errno;
